@@ -30,6 +30,7 @@ from test.picardtestcase import (
 from picard import config
 from picard.album import Album
 from picard.mbjson import (
+    _decamelcase,
     artist_to_metadata,
     countries_from_node,
     get_score,
@@ -52,6 +53,7 @@ settings = {
     "standardize_releases": False,
     "translate_artist_names": True,
     "standardize_instruments": True,
+    "release_ars": True,
     "preferred_release_countries": [],
     "artist_locale": 'en'
 }
@@ -85,6 +87,38 @@ class ReleaseTest(MBJSONTest):
         self.assertEqual(m['label'], 'Harvest')
         self.assertEqual(m['musicbrainz_albumartistid'], '83d91898-7763-47d7-b03b-b92132375c47')
         self.assertEqual(m['musicbrainz_albumid'], 'b84ee12a-09ef-421b-82de-0441a926375b')
+        self.assertEqual(m['producer'], 'Hipgnosis')
+        self.assertEqual(m['releasecountry'], 'GB')
+        self.assertEqual(m['releasestatus'], 'official')
+        self.assertEqual(m['script'], 'Latn')
+        self.assertEqual(m['~albumartists'], 'Pink Floyd')
+        self.assertEqual(m['~albumartists_sort'], 'Pink Floyd')
+        self.assertEqual(m['~releaselanguage'], 'eng')
+        self.assertEqual(m.getall('~releasecountries'), ['GB', 'NZ'])
+        self.assertEqual(a.genres, {
+            'genre1': 6, 'genre2': 3,
+            'tag1': 6, 'tag2': 3})
+        for artist in a._album_artists:
+            self.assertEqual(artist.genres, {
+                'british': 2,
+                'progressive rock': 10})
+
+    def test_release_without_release_relationships(self):
+        config.setting['release_ars'] = False
+        m = Metadata()
+        a = Album("1")
+        release_to_metadata(self.json_doc, m, a)
+        self.assertEqual(m['album'], 'The Dark Side of the Moon')
+        self.assertEqual(m['albumartist'], 'Pink Floyd')
+        self.assertEqual(m['albumartistsort'], 'Pink Floyd')
+        self.assertEqual(m['asin'], 'b123')
+        self.assertEqual(m['barcode'], '123')
+        self.assertEqual(m['catalognumber'], 'SHVL 804')
+        self.assertEqual(m['date'], '1973-03-24')
+        self.assertEqual(m['label'], 'Harvest')
+        self.assertEqual(m['musicbrainz_albumartistid'], '83d91898-7763-47d7-b03b-b92132375c47')
+        self.assertEqual(m['musicbrainz_albumid'], 'b84ee12a-09ef-421b-82de-0441a926375b')
+        self.assertEqual(m['producer'], '')
         self.assertEqual(m['releasecountry'], 'GB')
         self.assertEqual(m['releasestatus'], 'official')
         self.assertEqual(m['script'], 'Latn')
@@ -174,6 +208,19 @@ class RecordingTest(MBJSONTest):
         self.assertEqual(m['performer:acoustic guitar'], 'Ed Sheeran')
 
 
+class RecordingInstrumentalTest(MBJSONTest):
+
+    filename = 'recording_instrumental.json'
+
+    def test_recording(self):
+        m = Metadata()
+        t = Track('1')
+        recording_to_metadata(self.json_doc, m, t)
+        self.assertIn('instrumental', m.getall('~performance_attributes'))
+        self.assertEqual(m['language'], 'zxx')
+        self.assertNotIn('lyricist', m)
+
+
 class NullRecordingTest(MBJSONTest):
 
     filename = 'recording_null.json'
@@ -205,6 +252,12 @@ class RecordingCreditsTest(MBJSONTest):
         self.assertNotIn('performer:solo', m)
         self.assertEqual(m['performer:solo vocals'], 'Anni-Frid Lyngstad')
 
+    def test_recording_instrument_decamelcase(self):
+        m = Metadata()
+        t = Track("1")
+        recording_to_metadata(self.json_doc, m, t)
+        self.assertEqual(m['performer:ewi'], 'Michael Brecker')
+
 
 class TrackTest(MBJSONTest):
 
@@ -217,7 +270,6 @@ class TrackTest(MBJSONTest):
         self.assertEqual(m['title'], 'Speak to Me')
         self.assertEqual(m['musicbrainz_recordingid'], 'bef3fddb-5aca-49f5-b2fd-d56a23268d63')
         self.assertEqual(m['musicbrainz_trackid'], 'd4156411-b884-368f-a4cb-7c0101a557a2')
-        self.assertEqual(m['title'], 'Speak to Me')
         self.assertEqual(m['~length'], '1:08')
         self.assertEqual(m['tracknumber'], '1')
         self.assertEqual(m['~musicbrainz_tracknumber'], 'A1')
@@ -225,6 +277,19 @@ class TrackTest(MBJSONTest):
         self.assertEqual(m['~recordingtitle'], 'Speak to Me')
 
 
+class PregapTrackTest(MBJSONTest):
+
+    filename = 'track_pregap.json'
+
+    def test_track(self):
+        t = Track("1")
+        m = t.metadata
+        track_to_metadata(self.json_doc, t)
+        self.assertEqual(m['title'], 'Lady')
+        self.assertEqual(m['tracknumber'], '0')
+        self.assertEqual(m['~musicbrainz_tracknumber'], '0')
+
+
 class NullTrackTest(MBJSONTest):
 
     filename = 'track_null.json'
@@ -371,3 +436,12 @@ class GetScoreTest(PicardTestCase):
 
     def test_get_score_no_score(self):
         self.assertEqual(1.0, get_score({}))
+
+
+class DecamelcaseTest(PicardTestCase):
+    def test_decamelcase(self):
+        self.assertEqual('foo bar', _decamelcase('foo bar'))
+        self.assertEqual('Foo Bar', _decamelcase('Foo Bar'))
+        self.assertEqual('Foo Bar', _decamelcase('FooBar'))
+        self.assertEqual('Foo BAr', _decamelcase('FooBAr'))
+        self.assertEqual('EWI', _decamelcase('EWI'))
