@@ -38,6 +38,7 @@ from test.picardtestcase import PicardTestCase
 from picard import util
 from picard.const.sys import IS_WIN
 from picard.util import (
+    extract_year_from_date,
     find_best_match,
     imageinfo,
     limited_join,
@@ -77,6 +78,25 @@ class ReplaceWin32IncompatTest(PicardTestCase):
                              "c:\\test\\te\"st2")
 
 
+class ExtractYearTest(PicardTestCase):
+    def test_string(self):
+        self.assertEqual(extract_year_from_date(""), None)
+        self.assertEqual(extract_year_from_date(2020), None)
+        self.assertEqual(extract_year_from_date("2020"), 2020)
+        self.assertEqual(extract_year_from_date('2020-02-28'), 2020)
+        self.assertEqual(extract_year_from_date('2015.02'), 2015)
+        self.assertEqual(extract_year_from_date('2015; 2015'), None)
+        # test for the format as supported by ID3 (https://id3.org/id3v2.4.0-structure): yyyy-MM-ddTHH:mm:ss
+        self.assertEqual(extract_year_from_date('2020-07-21T13:00:00'), 2020)
+
+    def test_mapping(self):
+        self.assertEqual(extract_year_from_date({}), None)
+        self.assertEqual(extract_year_from_date({'year': 'abc'}), None)
+        self.assertEqual(extract_year_from_date({'year': '2020'}), 2020)
+        self.assertEqual(extract_year_from_date({'year': 2020}), 2020)
+        self.assertEqual(extract_year_from_date({'year': '2020-02-28'}), None)
+
+
 class SanitizeDateTest(PicardTestCase):
 
     def test_correct(self):
